import ox.cads.testing._
import ox.cads.collection._

object BadHashSetTest{
  var iters = 200 // # iterations per worker
  val MaxItem = 8

  // probabilities of different operations
  val (readProb, addProb, delProb) = (0.4, 0.3, 0.3) // ****(0.5, 0.5, 0.0) //

  val p = 4 // # workers

  type CSet = ox.cads.collection.Set[BadHashString]
  type SSet = scala.collection.immutable.Set[BadHashString]

  // Operations upon sequential sets
  def seqContains(item: BadHashString)(s: SSet) : (Boolean, SSet) = 
    (s.contains(item), s)
  def seqAdd(item: BadHashString)(s: SSet) : (Boolean, SSet) =
    (!s.contains(item), s+item)
  def seqRemove(item: BadHashString)(s: SSet) : (Boolean, SSet) = 
    (s.contains(item), s-item)

  // A worker
  def worker(me: Int, log: GenericThreadLog[SSet,CSet]) = {
    val random = new scala.util.Random
    for(i <- 0 until iters){
      val item = new BadHashString(random.nextInt(MaxItem).toString)
      val rand = random.nextFloat
      if(rand < readProb) // contains
        log(_.contains(item), "contains "+item, seqContains(item))
      else if(rand < readProb+addProb) // add
        log(_.add(item), "add "+item, seqAdd(item))
      else // remove
        log(_.remove(item), "remove "+item, seqRemove(item))
    }
  }

  val usage = 
    """|scala -J-Xmx10g SetTest [--reps n | --untilError] [--iters n] 
       |[--fine | --optimistic | --lazy | --lockFree | --lazySkipList | --lockFreeSkipList]""".stripMargin

  def main(args: Array[String]) = {
    var reps = 5000 // # repetitions
    var setType = "coarse"
    var timing = false
    // Parse arguments
    var i = 0
    while(i < args.length){
      if(args(i) == "--timing"){ timing = true; i += 1 }
      else if(args(i) == "--iters"){ iters = args(i+1).toInt; i += 2 }
      else if(args(i) == "--reps"){ reps = args(i+1).toInt; i += 2 }
      else if(args(i) == "--untilError"){ reps = -1; i += 1 }
      else if(args(i) == "--fine"){ setType = "fine"; i += 1 }
      else if(args(i) == "--optimistic"){ setType = "optimistic"; i += 1 }
      else if(args(i) == "--lazy"){ setType = "lazy"; i += 1 }
      else if(args(i) == "--lockFree"){ setType = "lockFree"; i += 1 }
      else if(args(i) == "--lazySkipList"){ setType = "lazySkipList"; i += 1 }
      else if(args(i) == "--collisionResistantLazySkipList"){ 
	setType = "collisionResistantLazySkipList"; i += 1 
      }
      else if(args(i) == "--collisionResistantLockFreeSkipList"){ 
	setType = "collisionResistantLockFreeSkipList"; i += 1 
      }
      else if(args(i) == "--lockFreeSkipList"){ 
	setType = "lockFreeSkipList"; i += 1 
      }
      else if(args(i) == "--lockFreeSkipListFaulty"){ 
	setType = "lockFreeSkipListFaulty"; i += 1 
      }      else sys.error(usage)
    }

    // Run tests
    val t0 = java.lang.System.nanoTime
    var r = 0; var done = false
    while(r != reps && !done){
      val seqSet : SSet = scala.collection.immutable.Set[BadHashString]()
      val concSet : CSet = 
	if (setType == "coarse") new ox.cads.collection.CoarseListSet[BadHashString]
	else if (setType == "fine") new FineListSet[BadHashString]
	else if (setType == "optimistic") new OptimisticListSet[BadHashString]
	else if (setType == "lazy") new LazyListSet[BadHashString]
	else if (setType == "lockFree") new LockFreeListSet[BadHashString]
	else if (setType == "lazySkipList") new LazySkipList[BadHashString]
	else if (setType == "lockFreeSkipListFaulty") 
	  new LockFreeSkipListFaulty[BadHashString]
	else if (setType == "collisionResistantLazySkipList") 
	  new CollisionResistantLazySkipList[BadHashString]
	else if (setType == "collisionResistantLockFreeSkipList") 
	  new CollisionResistantLockFreeSkipList[BadHashString]
	else if (setType == "lockFreeSkipList") new LockFreeSkipList[BadHashString]
	else sys.error("bad set type")
      val tester = LinearizabilityTester.JITGraph[SSet, CSet](
	seqSet, concSet, p, worker, iters)
      // new DFSLinearizabilityTester(seqSet, concSet, p, worker _, p*iters)
      done = (tester() < 0)
      if(r%50 == 0 && !timing) print(".")
      r += 1
    }
    val t1 = java.lang.System.nanoTime
    if(timing) println(t1-t0) else println("\n"+(t1-t0)/1000000)
  }
}
