/** An experiment comparing the three testers on a map, varying the 
  * number of workers performing operations. */

import ox.cads.experiments.{Experiments,Graphs}

object ErrorExperiment{
  val cmd0 = "scala -J-Xmx12g "
  val cmd1 = " --timing --untilError"

  // Array giving names of tests, and flags to pass to the tester
  val errors = Array( // ***
    //("LockFreeSkipListFaulty", "BadHashSetTest --lockFreeSkipListFaulty"),
    // ("LvdPWFaulty", 
    //  "MapTest --JIT-JIT-Comp --LvdPWFaulty --maxKey 2 -p 4 --iters 420"),
    // ("LvdPWFaultyResize", 
    //  "MapTest --JIT-JIT-Comp --maxKey 32 --LvdPWResizeMapFaulty --iters 200")
    ("LockFreeReadShardedFaulty", 
     "MapTest --JIT-JIT-Comp --LockFreeReadShardedFaulty  --maxKey 2")
    //("TimestampTest", "TimestampTest")
  )
   
      // ("LvdPWBadHash", "MapTest --comp --LvdPWBadHash --badHash"), 
      // ("LockFreeSkipList", "BadHashSetTest --lockFreeSkipList"),

  val Billion = 1000000000

  /** Parameters defining statistical analysis of experiments: repeat until 
    * 95% confidence half-interval is at most 1% of mean or until 50 
    * repetitions done. */
  val params = new Experiments.Params(50, 0.05, 0.01) 
  // val params = new Experiments.Params(5, 0.05, 0.1) // ***

  def main(args: Array[String]) = {
    // Array for results, in millis 
    val results = new Array[(Double,Double)](errors.length)

    for(i <- 0 until errors.length){
      val (name, flags) = errors(i)
      println(name)
      val cmd = cmd0+flags+cmd1
      // println(cmd)
      // A single measurement, giving time in nanos
      def measure : Double = {
	val time = Experiments.timeProc(cmd)
	print(time/1000000+" ")
	time.toDouble
      }
      val (m, s) = Experiments.iterateMeasurement(measure, params)
      println; println(m/1000000,s/1000000) // in millis
      results(i) = (m/Billion,s/Billion)
    }

    for(i <- 0 until errors.length)
      println(errors(i)._1+"\t"+results(i)._1+"\t"+results(i)._2)
   
    
    // // Produce graphs
    // val options = 
    //   Array[String](
    //     "title = \\begin{tabular}{l}Comparison of linearizability testers \\\\"+
    // 	"for a map with different \\\\"+
    // 	"numbers of workers \\end{tabular}",
    // 	"xlabel = Number of workers", 
    //     "ylabel = Time (s)", 
    //     "legend pos = north west",
    //     "height = 0.6\\textheight", "width = 0.5\\textwidth"
    //   )
    // val graphString = 
    //   Graphs.makeLinearGraph(options, testers.map(_._1), ps, results)
    // println(graphString)
    // val fname = "workerExperiment.tex"
    // Graphs.writeStandAloneFile(fname, graphString)
  }
}
