import ox.cads.experiments.{Experiments,Graphs}

/** An experiment comparing the three testers on a map, varying the 
  * number of keys. */
object MapExperiment{
  // number of operations per run; number of runs
  val numIters = 1 << 11; val reps = 1 << 7

  // Array giving names of testers (for graphs) and the appropriate flags
  val testers =  // ***
    Array(      
      ("WG-Graph", "--WGGraph"), 
      ("JIT-Graph", "--JITGraph"), 
      // Competition; names give the tree search algorithm, then the graph 
      // search algorithm
      ("WG-WG-Competition", "--WG-WG-Comp"), 
      ("JIT-WG-Competition", "--JIT-WG-Comp"),
      ("WG-JIT-Competition",  "--WG-JIT-Comp"),
      ("JIT-JIT-Competition", "--JIT-JIT-Comp")
    )

  val cmd0 = 
    "scala -J-Xmx12g MapTest --timing --iters "+numIters+" --reps "+reps+" "

  val Billion = 1000000000

  // Sizes of key spaces
  val mapSizes = (0 to 10).map(1 << _).toArray

  /** Parameters defining statistical analysis of experiments: repeat until 
    * 95% confidence half-interval is at most 1% of mean or until 50
    * repetitions done. */
  val params = new Experiments.Params(50, 0.05, 0.01) 
  // val params = new Experiments.Params(5, 0.05, 0.1) // ***

  def main(args: Array[String]) = {
    // Array for results, in millis
    val results = 
      Array.ofDim[(Double,Double)](testers.length, mapSizes.length)

    for(i <- 0 until mapSizes.length){
      val size = mapSizes(i); println("map size = "+size)
      for(t <- 0 until testers.length){
	println(testers(t)._1)
	  val cmd = cmd0+testers(t)._2+" --maxKey "+size
	  // A single measurement, giving time in nanos
	  // println(cmd)
	  def measure : Double = {
	    val time = Experiments.timeProc(cmd)
	    print(time/1000000+" ")
	    time.toDouble
	  }
	  val (m, s) = Experiments.iterateMeasurement(measure, params)
          println(m/1000000,s/1000000) // in millis
          results(t)(i) = (m/Billion,s/Billion)
	}
      }

    // ox.cads.util.Profiler.report; sys.exit // ****
    // Produce graphs
    val options = 
      Array[String](
        "title = \\begin{tabular}{l}Comparison of linearizability testers \\\\"+
	"for a map with different \\\\"+
	"numbers of keys \\end{tabular}",
	"xlabel = Number of keys", 
        "ylabel = Time (s)", 
	"log basis x=2",
        "legend pos = north west",
        "height = 0.6\\textheight", "width = 0.5\\textwidth"
      )
    val graphString = 
      Graphs.makeLogXGraph(options, testers.map(_._1), mapSizes, results)
    println(graphString)
    val fname = "mapSize.tex"
    Graphs.writeStandAloneFile(fname, graphString)
  }
}
