import ox.cads.util.Profiler
import ox.cads.testing._
// import ox.cads.collection.ShardedMap
import scala.util.Random
// import scala.reflect.ClassTag

object MapTest{
  val testers0 = 
    List(
      // Tree search
      "WGTree", "JITTree", // "DFSJIT",
      // Graph search
      "WGGraph", "JITGraph", "BFSJIT",
      // Competition; tree-search algorithm name comes first
      "WG-WG-Comp", "JIT-WG-Comp", "WG-JIT-Comp", "JIT-JIT-Comp"
    )
  // List of options for defining testers
  val testers = testers0.map("--" + _)

  val usage = (
    """|scala -J-Xmx10g MapTest
       | [""" + testers.mkString(" | ") + "]\n" +
    """| [--LvdPWFaulty | --LvdPWBadHash | --Sharded | --RSO]
       | [--sharedLog] [--timing] [--reps n | --untilError] [--maxKey n] 
       | [--iters n] [--badHash] [--maxConfigs n] [-p p]""").stripMargin

  def main(args: Array[String]) = {
    // Parse arguments
    var p = 4 // # workers
    var iters = 200 // # iterations
    var maxKey = 20 // maximum key size
    var timing = false; var i = 0
    var reps = 500 // Number of repetitions
    var tester = "WGGraph" // type of linearizability tester
    var concMapType = "Sharded" // type of concurrent map
    var badHash = false // are we using bad hashes
    var maxConfigs = -1L // maximum # configurations
    var tsLog = true // use timestamp-based log?
    while(i < args.length){
      if(args(i) == "--timing"){ timing = true; i += 1 }
      else if(testers.contains(args(i))){ tester = args(i).drop(2); i += 1 }
      else if(args(i) == "--iters"){ iters = args(i+1).toInt; i += 2 }
      else if(args(i) == "--reps"){ reps = args(i+1).toInt; i += 2 }
      else if(args(i) == "--untilError"){ reps = -1; i += 1 }
      else if(args(i) == "--maxKey"){ maxKey = args(i+1).toInt; i += 2 }
      else if(args(i) == "--maxConfigs"){ maxConfigs = args(i+1).toLong; i += 2 }
      else if(args(i) == "--badHash"){ badHash = true; i += 1 } 
      else if(args(i) == "--sharedLog"){ tsLog = false; i += 1 }
      // choice of hash map
      else if(args(i) == "--LvdPWFaulty"){ 
	concMapType = "LvdPWFaulty"; i += 1 
      }
      else if(args(i) == "--LvdPWResizeMapFaulty"){ 
	concMapType = "LvdPWResizeMapFaulty"; i += 1 
      }
      else if(args(i) == "--LockFreeReadShardedFaulty"){ 
	concMapType = "LockFreeReadShardedFaulty"; i += 1 
      }
      else if(args(i) == "--LvdPWBadHash"){ 
	concMapType = "LvdPWBadHash"; i += 1 
      }
      else if(args(i) == "--RSO"){ concMapType = "RSO"; i += 1 }
      // else if(args(i) == "--LvdPW"){ cMapType = "LvdPW"; i += 1 }
      // else if(args(i) == "--LvdPWdel"){	cMapType = "LvdPWdel"; i += 1 }
      // else if(args(i) == "--LvdPWResize"){ cMapType = "LvdPWResize"; i += 1 }
      // else if(args(i) == "--SimpleSharded"){ 
      // 	cMapType = "SimpleSharded"; i += 1 
      // }
      else if(args(i) == "--Sharded"){ concMapType = "Sharded"; i += 1 }
      // else if(args(i) == "--LockFreeReadSharded"){ 
      // 	cMapType = "LockFreeReadSharded"; i += 1 
      // }
      // else if(args(i) == "--LockFreeReadAtomicSharded"){ 
      // 	cMapType = "LockFreeReadAtomicSharded"; i += 1 
      // }
      else if(args(i) == "-p"){ p = args(i+1).toInt; i += 2 }
      else{ println("Invalid argument; usage:\n"+usage); sys.exit }
    }

    // Run the test
    val t0 = java.lang.System.nanoTime
    val result = 
      if(!badHash){
	def makeDatum(size: Int, random: Random) = 
	  random.nextInt(size).toString
	val test = 
	  new MapLinTest[String](
	    makeDatum, "X", p, iters, reps, maxKey, 
	    concMapType, tester, tsLog, timing)
	test()
      }
      else{
	def makeDatum(size: Int, random: Random) = 
	  new BadHashString(random.nextInt(size).toString)
	val test = 
	  new MapLinTest[BadHashString](
	    makeDatum, new BadHashString("X"), p, iters, reps, maxKey, 
	    concMapType, tester, tsLog, timing)
	test()
      }
    val t1 = java.lang.System.nanoTime
    if(result == Solver.OutOfSteam) println(Long.MaxValue)
    else if(timing) println(t1-t0) else println("\n"+(t1-t0)/1000000)
    Profiler.report
  }
  

}
