import ox.cads.experiments.{Experiments,Graphs}

/** An experiment comparing testers on a queue, varying the probability of
  * enqueueing.  Most testers tend to run out of memory when the 
  * probability of enqueueing is more than about 0.4.  
  */
object QueueExperiment{
  // number of operations per run; number of runs
  val numIters = 1 << 11; val reps = 1 << 7

  // Array giving names of testers (for graphs), the appropriate flags, and
  // the max probability that seems to work.
  val testers =  // ***
    Array(("WG-Graph", "--WGGraph"), ("JIT-Graph", "--JITGraph"),
	  ("JIT-JIT-Competition", "--JIT-JIT-Comp"), 
	  ("Queue", "--QLT") )
    // ("Wing \\& Gong", "--WG"), 

  // val cp = "/users/gavinl/Scala/CADS:/users/gavinl/Scala/CADS/Tests/testing"
  val cmd0 = 
    "scala -J-Xmx12g QueueTest --timing --iters "+numIters+" --reps "+reps+" "

  val Billion = 1000000000
  val ErrorSignal = ox.cads.experiments.Experiments.ErrorSignal

  // Probabilities of doing an enqueue
  // val enqueueProbs = (0.4 to 0.98 by 0.02).toArray
  val enqueueProbs = (0.02 to 0.98 by 0.02).toArray

  /** Parameters defining statistical analysis of experiments: repeat until 95% 
    * confidence half-interval is at most 1% of mean or until 50 repetitions
    * done. */
  val params = new Experiments.Params(50, 0.05, 0.01) 
  // val params = new Experiments.Params(5, 0.05, 0.1) // ***

  def main(args: Array[String]) = {
    // Array for results, in seconds
    val results = 
      Array.ofDim[(Double,Double)](testers.length, enqueueProbs.length)

    for(i <- 0 until enqueueProbs.length){
      val ep = enqueueProbs(i); println("enqueue prob = "+ep)
      for(t <- 0 until testers.length){
	//if(ep <= testers(t)._3)
	  println(testers(t)._1)
	  if(i > 0 && results(t)(i-1)._1 == ErrorSignal){
	    println("Skipping"); results(t)(i) = (ErrorSignal,0)
	  }
	  else{
	    val cmd = cmd0+testers(t)._2+" --enqueueProb "+ep
	    // A single measurement, giving time in nanos
	    // println(cmd)
	    def measure : Double = {
	      val time = Experiments.timeProc(cmd)
	      print(time/1000000+" ")
	      time.toDouble
	    }
	    val (m, s) = Experiments.iterateMeasurement(measure, params)
            println(m/1000000,s/1000000) // in millis
            if(m != ErrorSignal) results(t)(i) = (m/Billion,s/Billion)
	    else results(t)(i) = (ErrorSignal, 0)
	      // results(t)(i) = (m/Billion,s/Billion)
	  }
      }
	//else println("Skipping "+testers(t)._1+" with probability "+ep)
    } // end of outer for loop

    // Produce graphs
    val options = 
      Array[String](
        "title = \\begin{tabular}{l}Comparison of linearizability testers \\\\"+
	"for a queue with different \\\\"+
	"probabilities of enqueueing \\end{tabular}",
	"xlabel = Probability of enqueueing", 
        "ylabel = Time (s)", 
	"xmin = 0.0, xmax = 1.0", 
        "legend pos = north east",
        "height = 0.6\\textheight", "width = 0.9\\textwidth"
      )
    val graphString = 
      Graphs.makeLinearGraph(options, testers.map(_._1), enqueueProbs, results)
    println(graphString)
    val fname = "queueEnqueueProb.tex"
    Graphs.writeStandAloneFile(fname, graphString)
  }
}
