
import java.util.concurrent.atomic.AtomicInteger
import ox.cads.atomic.AtomicPair

/** An implementation of the incorrect array-based queue of Shann, Huang and 
  * Chen. 
  *
  * @tparam A the type of data held in the queue.
  * @param capacity the capacity of the queue. */


class SHCQueue[A](capacity: Int) extends ox.cads.collection.Queue[A]{
  /** The index of the first element of the queue (interpreted mod capacity). */
  private val front = new AtomicInteger(0)

  /** The index of the first free slot in the array (interpreted mod 
    * capacity). */
  private val rear = new AtomicInteger(0)

  /** Array holding the array entries. */
  private val q = Array.fill(capacity)(new AtomicPair(None: Option[A], 0))

  /* Let f be the first value >= front such that q(f%capacity) is filled 
   * (i.e. has non-None first entry).  Let r be the first value >= rear such
   * that q(r%capacity) is not filled.  Then this represents the queue with
   * contents [ q(i%capacity)._1.get | i <- [f..r) ].
   */

  /** Enqueue v, retrying until successful. */
  def enqueue(v: A) = {
    print("e")
    var done = false
    do{
      val myRear = rear.get
      val (x,c) = q(myRear%capacity).get
      // Check rear not changed and array not full
      if(myRear == rear.get && myRear != front.get+capacity){
	if(x == None){ 
	  // try to write date in array; following CAS is lin pt if successful
	  if(q(myRear%capacity).compareAndSet((x,c), (Some(v),c+1))){
	    rear.compareAndSet(myRear, myRear+1) // advance rear
	    done = true
	  }
	}
	else // x == None; help advance rear
	  rear.compareAndSet(myRear, myRear+1) // help advance rear
      }
    } while(!done)
  }

  /** Dequeue a value, retrying until successful. */
  def dequeue : Option[A] = { 
    print("d")
    while(true){
      val myFront = front.get
      val (x,c) = q(myFront%capacity).get
      // val myRear = rear.get
      // Check front not changed and queue not empty
      if(myFront == front.get){
	if(myFront == rear.get){ } // return None
	else if(x != None){
	  // Try to clear slot; following CAS is lin pt if successful
	  if(q(myFront%capacity).compareAndSet((x,c), (None,c+1))){
	    front.compareAndSet(myFront, myFront+1)
	    return x
	  }
	}
	else // x == None 
	  front.compareAndSet(myFront, myFront+1) // help advance front
      }
    }
    sys.error("unreachable")
  }
}
 
