import ox.cads.testing._
import ox.cads.collection._

object SetTest{
  var iters = 200 // # iterations per worker
  val MaxItem = 4

  // probabilities of different operations
  val (readProb, addProb, delProb) =  (0.4, 0.3, 0.3) // (0.0, 1.0, 0.0) 

  val p = 4 // # workers

  type CSet = ox.cads.collection.Set[Int]
  type SSet = scala.collection.immutable.Set[Int]

  // Operations upon sequential sets
  def seqContains(item: Int)(s: SSet) : (Boolean, SSet) = 
    (s.contains(item), s)
  def seqAdd(item: Int)(s: SSet) : (Boolean, SSet) =
    (!s.contains(item), s+item)
  def seqRemove(item: Int)(s: SSet) : (Boolean, SSet) = 
    (s.contains(item), s-item)

  // A worker
  def worker(me: Int, log: GenericThreadLog[SSet,CSet]) = {
    val random = new scala.util.Random
    for(i <- 0 until iters){
      val item = random.nextInt(MaxItem)
      val rand = random.nextFloat
      if(rand < readProb) // contains
        log(_.contains(item), "contains "+item, seqContains(item))
      else if(rand < readProb+addProb) // add
        log(_.add(item), "add "+item, seqAdd(item))
      else // remove
        log(_.remove(item), "remove "+item, seqRemove(item))
    }
  }

  val usage = 
    """|scala -J-Xmx10g SetTest [--reps n | --untilError] [--iters n] 
       |[--fine | --optimistic | --lazy | --lockFree | --lazySkipList | --lockFreeSkipList]""".stripMargin

  def main(args: Array[String]) = {
    var reps = 5000 // # repetitions
    var setType = "coarse"
    // Parse arguments
    var i = 0
    while(i < args.length){
      if(args(i) == "--iters"){ iters = args(i+1).toInt; i += 2 }
      else if(args(i) == "--reps"){ reps = args(i+1).toInt; i += 2 }
      else if(args(i) == "--untilError"){ reps = -1; i += 1 }
      else if(args(i) == "--fine"){ setType = "fine"; i += 1 }
      else if(args(i) == "--optimistic"){ setType = "optimistic"; i += 1 }
      else if(args(i) == "--lazy"){ setType = "lazy"; i += 1 }
      else if(args(i) == "--lockFree"){ setType = "lockFree"; i += 1 }
      else if(args(i) == "--collisionResistantLazySkipList"){ 
        setType = "collisionResistantLazySkipList"; i += 1 
      }
      else if(args(i) == "--lazySkipList"){ setType = "lazySkipList"; i += 1 }
      else if(args(i) == "--lockFreeSkipList"){ 
        setType = "lockFreeSkipList"; i += 1 
      }
      else sys.error(usage)
    }

    // Run tests
    for(r <- 0 until reps){
      val seqSet : SSet = scala.collection.immutable.Set[Int]()
      val concSet : CSet = 
        if (setType == "coarse") new ox.cads.collection.CoarseListSet[Int]
        else if (setType == "fine") new FineListSet[Int]
        else if (setType == "optimistic") new OptimisticListSet[Int]
        else if (setType == "lazy") new LazyListSet[Int]
        else if (setType == "lockFree") new LockFreeListSet[Int]
        else if (setType == "lazySkipList") new LazySkipList[Int]
        else if (setType == "collisionResistantLazySkipList") 
          new CollisionResistantLazySkipList[Int]
        else if (setType == "lockFreeSkipList") new LockFreeSkipList[Int]
        else sys.error("bad set type")
      val tester = LinearizabilityTester.JITGraph[SSet, CSet](
	seqSet, concSet, p, worker, iters)
      //  new DFSLinearizabilityTester(seqSet, concSet, p, worker _, p*iters)
      if(tester() < 0) sys.exit
      if(r%50 == 0) print(".")
    }
    println
  }
}
