import ox.cads.testing._

object TimestampTest{
  private var iters = 200 // # iterations per worker
  private val p = 4 // # workers
  private var reps =  5000
  private var volatile = false // do we use a volatile variable?
  
  private type S = ImmutableVariable
  private type V = Variable
  private type MyLog = Log[S,Variable]

  /** A worker using a GenericThreadLog. */ 
  private def worker(me: Int, log: GenericThreadLog[S, Variable]) = {
    val random = new scala.util.Random

    for(_ <- 0 until iters){
      if(random.nextFloat < 0.6) 
	log(_.read, "read", _.read)
      else{
	val x = random.nextInt(10)
	log(_.write(x), "write("+x+")", _.write(x))
      }
    }
  }

  def main(args: Array[String]) = {
    // Parse arguments
    var timing = false
    var i = 0
    while(i < args.length){
      if(args(i) == "--timing"){ timing = true; i += 1 }
      else if(args(i) == "--volatile"){ volatile = true; i += 1 }
      else if(args(i) == "--iters"){ iters = args(i+1).toInt; i += 2 }
      else if(args(i) == "--reps"){ reps = args(i+1).toInt; i += 2 }
      else if(args(i) == "--untilError"){ reps = -1; i += 1 }
      else{ println("Illegal argument"); sys.exit }
    }


    // Now run the tests
    val t0 = java.lang.System.nanoTime
    var r = 0L; var result = 2
    while(r != reps && result > 0){
      // Run workers
      val variable =  // concurrent datatype
       	if(volatile) new VolatileVariable else new Variable
      val s = new ImmutableVariable(0) // specification object 
      val tester = LinearizabilityTester.JITGraph[S, Variable](
       	s, variable, p, worker, iters, true)
      result = tester()      
      // if(result < 0){ println(i); sys.exit }
      if(i%100 == 0) print(".")
      r += 1
    }
    val t1 = java.lang.System.nanoTime
    if(result == Solver.OutOfSteam) println(Long.MaxValue)
    else if(timing) println(t1-t0) else println("\n"+(t1-t0)/1000000)
  }


}
