import ox.cads.testing._

object TimestampTest{
  private var iters = 200 // # iterations per worker
  private val p = 4 // # workers
  private var reps =  5000
  private var volatile = false // do we use a volatile variable?
  
  private type S = ImmutableVariable
  private type V = Variable
  private type MyLog = Log[S,Variable]

  /** A worker using a GenericThreadLog. */ 
  private def worker(me: Int, log: GenericThreadLog[S, Variable]) = {
    val random = new scala.util.Random

    for(_ <- 0 until iters){
      if(random.nextFloat < 0.6) 
	log(_.read, "read", _.read)
      else{
	val x = 5+random.nextInt(5)
	log(_.write(x), "write("+x+")", _.write(x))
      }
    }
  }

  def main(args: Array[String]) = {
    // Parse arguments
    var i = 0
    while(i < args.length){
      if(args(i) == "--volatile"){ volatile = true; i += 1 }
      else if(args(i) == "--iters"){ iters = args(i+1).toInt; i += 2 }
      else if(args(i) == "--reps"){ reps = args(i+1).toInt; i += 2 }
    }

    for(i <- 0 until reps){
      // Run workers
      val variable =  // concurrent datatype
       	if(volatile) new VolatileVariable else new Variable
      val s = new ImmutableVariable(0) // specification object 
      val tester = LinearizabilityTester.JITGraph[S, Variable](
       	s, variable, p, worker, iters, true)
      val result = tester()      
      if(result < 0){ println(i); sys.exit }
      if(i%100 == 0) print(".")
    }
    println
  }


}
