import ox.cads.experiments.{Experiments,Graphs}

/** An experiment comparing the three testers on a map, varying the 
  * number of workers performing operations. */
object WorkerExperiment{
  val maxP = 8 // Maximum number of workers. 
  val minP = 1 // Minimum number of workers. 
  val ps = (minP to maxP).toArray // numbers of workers 
  val reps = 1 << 8  // number of runs
  val totalIters = 105*64 // total number of iterations; each worker does 1/p.

  var useMap = true

  // Array giving names of testers (for graphs) and the appropriate flags
  def testers = 
    Array(("WG-Graph", "--WGGraph"), ("JIT-Graph", "--JITGraph"))++
    (if(useMap) Array[(String,String)]() else Array(("Queue", "--QLT")))++
    Array(
      // ("WG-WG-Competition", "--WG-WG-Comp"),   // **
      // ("JIT-WG-Competition", "--JIT-WG-Comp"),
      ("WG-JIT-Competition",  "--WG-JIT-Comp"),
      ("JIT-JIT-Competition", "--JIT-JIT-Comp"))
 
  def cmd0 = 
    "scala -J-Xmx12g "+(if(useMap) "MapTest" else "QueueTest")+
    " --timing --reps "+reps+" "

  val Billion = 1000000000

  /** Parameters defining statistical analysis of experiments: repeat until 
    * 95% confidence half-interval is at most 1% of mean or until 50 
    * repetitions done. */
  val params = new Experiments.Params(50, 0.05, 0.01) 
  // val params = new Experiments.Params(5, 0.05, 0.1) // ***

  def main(args: Array[String]) = {
    for(i <- 0 to 1){
      useMap = i==0
      // Array for results, in millis
      val name = if(useMap) "map" else "queue"
      val results = 
	Array.ofDim[(Double,Double)](testers.length, maxP+1-minP)

      for(p <- ps){
	println("p = "+p+"; "+name)
	for(t <- 0 until testers.length){
	  println(testers(t)._1)
	  val cmd = cmd0+testers(t)._2+" -p "+p+" --iters "+(totalIters/p)
	  // A single measurement, giving time in nanos
	  // println(cmd)
	  def measure : Double = {
	    val time = Experiments.timeProc(cmd)
	    print(time/1000000+" ")
	    time.toDouble
	  }
	  val (m, s) = Experiments.iterateMeasurement(measure, params)
          println(m/1000000,s/1000000) // in millis
          results(t)(p-minP) = (m/Billion,s/Billion)
	}
      }

      // Produce graphs
      val options = 
	Array[String](
          "title = \\begin{tabular}{l}Comparison of linearizability testers \\\\"+
	  "for a "+name+" with different \\\\"+
	  "numbers of workers \\end{tabular}",
	  "xlabel = Number of workers", 
          "ylabel = Time (s)", 
          "legend pos = north west",
          "height = 0.6\\textheight", "width = 0.5\\textwidth"
	)
      val graphString = 
	Graphs.makeLinearGraph(options, testers.map(_._1), ps, results)
      println(graphString)
      val fname = name+"WorkerExperiment.tex"
      Graphs.writeStandAloneFile(fname, graphString)
    }
  }
}
