import ox.cads.experiments.{Experiments,Graphs}

/** An experiment  varying the number of workers performing operations, 
  * trying to find the error with LvdPWFaulty. */
object WorkersErrorExperiment{
  val minP = 2  // Minimum number of workers // ***
  val maxP = 2 // Maximum number of workers. 
  val ps = (minP to maxP).toArray // numbers of workers
  // val reps = 1 << 9  // number of runs
  // Total number of iterations; each worker does 1/p.  The number is chosen
  // to be divisible by each number from 1 to 8, and approximately equal to
  // 2^13
  val totalIters = 105*16*5

  val cmd0 = 
    "scala -J-Xmx12g MapTest --JIT-JIT-Comp --LvdPWFaulty --maxKey 2 --timing --untilError "

  val Million = 1000000
  val Billion = 1000000000

  /** Parameters defining statistical analysis of experiments: repeat until 
    * 95% confidence half-interval is at most 1% of mean or until 50 
    * repetitions done. */
  // val params0 = new Experiments.Params(50, 0.05, 0.01) 
  // val paramsStrict = new Experiments.Params(200, 0.05, 0.01) 
  // val params = new Experiments.Params(5, 0.05, 0.1) // ***
  val params = new Experiments.Params(50, 0.05, 0.01) // ***

  def main(args: Array[String]) = {
    // Array for results, in millis
    val results = Array.ofDim[(Double,Double)](ps.length)

    for(i <- 0 until ps.length){
      val p = ps(i); println("p = "+p)
      val cmd = cmd0+" -p "+p+" --iters "+(totalIters/p)
      // A single measurement, giving time in nanos
      println(cmd)
      var count = 0
      def measure : Double = {
	val time = Experiments.timeProc(cmd)
	println(time/Million+"; count = "+count+"; p = "+p) // in milliseconds
	count += 1
	time.toDouble
      }
      // val params = if(p<4) params0 else paramsStrict
      val (m, s) = Experiments.iterateMeasurement(measure, params)
      println(m/Million,s/Million) // in milliseconds
      results(i) = (m/Billion,s/Billion) // in seconds
      Graphs.writeFile("tmp"+p, results(i).toString)	
    }

    // Produce graphs
    val options = 
      Array[String](
        "title = \\begin{tabular}{l}Comparison of time to find an \\\\"+
	"error, with different numbers of workers \\end{tabular}",
	"xlabel = Number of workers", 
        "ylabel = Time (s)", 
        "legend pos = north west",
        "height = 0.6\\textheight", "width = 0.5\\textwidth"
      )
    val graphString = 
      Graphs.makeLinearGraph(options, Array("JIT-JIT-Competition"), ps, Array(results))
    println(graphString)
    val fname = "workerErrorExperiment.tex"
    Graphs.writeStandAloneFile(fname, graphString)
    println("Results written to "+fname)
  } 
}
